package dev.peerat.mapping.providers.mysql;

import static dev.peerat.parser.java.builder.JavaBuilder.ofFile;
import static dev.peerat.parser.java.builder.JavaBuilder.ofClass;
import static dev.peerat.parser.java.builder.JavaBuilder.ofFunction;
import static dev.peerat.parser.java.builder.JavaBuilder.ofVariable;
import static dev.peerat.parser.java.builder.JavaBuilder.ofOperation;
import static dev.peerat.parser.java.builder.JavaBuilder.ofValue;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import dev.peerat.mapping.Ship;
import dev.peerat.mapping.providers.Provider;
import dev.peerat.mapping.providers.ProviderManager;
import dev.peerat.parser.ElementBuilder.Builder;
import dev.peerat.parser.Token;
import dev.peerat.parser.java.Class;
import dev.peerat.parser.java.Function;
import dev.peerat.parser.java.JavaFile;
import dev.peerat.parser.java.Variable;
import dev.peerat.parser.java.builder.JavaBuilder;
import dev.peerat.parser.java.builder.JavaClassBuilder;
import dev.peerat.parser.java.builder.JavaFunctionBuilder;
import dev.peerat.parser.java.operation.CatchOperation;
import dev.peerat.parser.java.operation.ReturnOperation;
import dev.peerat.parser.java.operation.TryOperation;
import dev.peerat.parser.java.value.MethodCallValue;
import dev.peerat.parser.java.value.Value;

public class MySQLProvider implements Provider{
	
	static{
		ProviderManager.getInstance().register("mysql", new MySQLProvider());
	}
	
	private List<String> treasures;
	private File repo;
	private List<MySQLRequestBuilder> requests;
	
	public MySQLProvider(){
		this.treasures = new ArrayList<>();
		this.requests = new ArrayList<>();
	}
	
	@Override
	public void init(File file) throws Exception{
		repo = new File(file, "/dev/peerat/mapping/mysql/MySQlRepository.java");
		repo.getParentFile().mkdirs();
		repo.createNewFile();
	}
	

	@Override
	public void mapTreasure(Class clazz){
		treasures.add(clazz.getName().getValue());
	}

	@Override
	public void mapBottle(Class clazz, Function function){
		ReturnOperation operation = (ReturnOperation) function.getElements().get(0);
		Value value = operation.getValue();
		if(value instanceof MethodCallValue){
			MethodCallValue call = (MethodCallValue)value;
			List<MethodCallValue> list = new LinkedList<>();
			while(call.base() instanceof MethodCallValue){
				list.add(0, call);
				call = (MethodCallValue) call.base();
			}
			list.add(0, call);
			MySQLRequestBuilder builder = new MySQLRequestBuilder(function, list);
			this.requests.add(builder);
		}
	}

	@Override
	public void mapUses(Class clazz, Function function){
		
	}


	@Override
	public void complete(){
		JavaClassBuilder clazz = ofClass("MySQlRepository");
		
		for(MySQLRequestBuilder request : this.requests){
			Function base = request.getFunction();
			JavaFunctionBuilder func = ofFunction(base.getReturnType().getValue(), "getter_"+base.getName().getValue()).setPublic();
			for(Variable params : base.getParameters()){
				func.parameter(ofVariable(params.getType().getValue(), params.getName().getValue(), params.getValue()));
			}
			clazz.element(
					func.element(
						ofOperation("try{"
								+ "ensureConnection();"
								+ "PreparedStatement stmt = con.prepareStatement();"
								+ "}catch(Exception e){"
								+ "e.printStackTrace();"
								+ "}")
					)
				);
		}
		
		JavaFile file = ofFile("dev.peerat.mapping.sql.MySQlRepository")
				.clazz(clazz)
				.build();
		try{
			Builder builder = new Builder();
			file.build(builder);
			BufferedWriter writer = new BufferedWriter(new FileWriter(this.repo));
			builder.build(writer);
			writer.flush();
			writer.close();
		}catch(Exception ex){
			throw new RuntimeException(ex);
		}
	}

	@Override
	public void register(Ship ship) throws Exception {
		
	}
	
	/*
	 *	try{
	 *		ensureConnection();
	 *		PreparedStatement stmt = con.prepareStatement(REQUEST);
	 *		//bind
	 *		//execute
	 *		//build object
	 *	}catch(Exception ex){
	 *		ex.printStackTrace();
	 *	} 
	 */
}
